#include <limits.h>

#include <QtCore>
#include <QtGui>

#include <KConfigDialog>
#include <KStandardDirs>
#include <KIcon>
#include <KIconLoader>

#include <Plasma/IconWidget>
#include <Plasma/Label>
#include <Plasma/LineEdit>

#include "openbrain.h"

using namespace std;

OpenBrain::OpenBrain(QObject *parent, const QVariantList &args) : Plasma::PopupApplet(parent, args), main_widget(0)
{
  setPopupIcon("preferences-desktop-user");
  setHasConfigurationInterface(true);
  setBackgroundHints(DefaultBackground);

  m_layout = 0;
  resize(290,150);
}

QGraphicsWidget *OpenBrain::graphicsWidget()
{
  if (!main_widget) {
    main_widget = new QGraphicsWidget(this);
    QSizePolicy expanding_policy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    QSizePolicy minimal_policy(QSizePolicy::Expanding, QSizePolicy::Maximum);
    m_layout = new QGraphicsLinearLayout(Qt::Vertical);
    m_inputLayout = new QGraphicsLinearLayout(Qt::Horizontal);
    m_inputLineEdit = new Plasma::LineEdit(this);
    m_response = new Plasma::Label(this);
    m_inputLayout->addItem(m_inputLineEdit);
    m_go = new Plasma::IconWidget(this);
    m_goAction = new QAction(KIcon("go-jump-locationbar"), QString(), this);
    m_go->setAction(m_goAction);
    m_go->setPreferredSize(m_go->sizeFromIconSize(IconSize(KIconLoader::Toolbar)));
    m_inputLayout->addItem(m_go);
    m_home = new Plasma::IconWidget(this);
    m_homeAction = new QAction(KIcon("go-home"), QString(), this);
    m_home->setAction(m_homeAction);
    m_home->setPreferredSize(m_home->sizeFromIconSize(IconSize(KIconLoader::Toolbar)));
    m_inputLayout->addItem(m_home);
    m_layout->addItem(m_inputLayout);
    m_layout->addItem(m_response);
    m_response->nativeWidget()->setSizePolicy(expanding_policy);
    m_response->nativeWidget()->setWordWrap(true);
    m_response->nativeWidget()->setTextInteractionFlags(Qt::NoTextInteraction);
    m_response->nativeWidget()->setAlignment(Qt::AlignCenter);    
    main_widget->setLayout(m_layout);
    main_widget->setPreferredSize(260, 200);
    setupConnections();
  }
  
  return main_widget;
}

void OpenBrain::init()
{
  KConfigGroup cg = config();

  QColor m_textColor = cg.readEntry("textcolor", QColor(Qt::black));
  QFont m_font = cg.readEntry("font", KGlobalSettings::generalFont());
  int customFontSize = cg.readEntry("customFontSize", m_font.pointSize());
  
  qDebug() << QString("openbrain's init() says kconfiggroup config()'s textColor is %1, font is %2, and fontsize is %3").arg(m_textColor.name()).arg(m_font.family()).arg(customFontSize);

  if (!QDir(QDir::homePath() + "/.openbrain").exists())
  {
	QProcess p(this);
	p.start("cp " + executeCommand("kde4-config --install data") + "/openbrain/openbrain_configuration " + QDir::homePath() + "/.openbrain -arf");
	if (!p.waitForFinished())
	{
	  QMessageBox::information(0, i18n("Error"), i18n("~/.openbrain did not exist, and copying the default 'brain' from %1 failed.", executeCommand("kde4-config --install data")));
	  return;
	}
  }
  loadBrain();
}

void OpenBrain::setupConnections()
{
  connect(m_goAction, SIGNAL(triggered()), this, SLOT(returnPressed()));
  connect(m_homeAction, SIGNAL(triggered()), this, SLOT(displayHome()));
  connect(m_inputLineEdit, SIGNAL(returnPressed()), this, SLOT(returnPressed()));
}

QString OpenBrain::executeCommand(const QString &commandStr)
{
/*  QString returnString("");
  QProcess proc;
  proc.start(commandStr);
  if (!proc.waitForFinished())
  {
    return commandStr + " could not be executed.";
  }
  while (!proc.atEnd())
  {
    QByteArray line = proc.readLine();
    if (line.length())
      line.truncate(line.length()-1);
    if (!line.isEmpty())
      returnString += (QString) line;
  }
  return returnString;*/
  QString returnString("");
  FILE *file = popen(commandStr.toAscii().data(), "r");
  if (!file)
  {
    return "";
  }
  char c = 0;
  while (c != EOF)
  {
    c = (char)getc(file);
    returnString += c;
  }
  fclose(file);
  return returnString.left(returnString.length() - 1).trimmed();
}

void OpenBrain::returnPressed()
{
  m_inputLineEdit->setText(m_inputLineEdit->text().replace("?", ""));
  m_inputLineEdit->setText(m_inputLineEdit->text().replace(".", ""));
  m_inputLineEdit->setText(m_inputLineEdit->text().replace(",", ""));
  m_inputLineEdit->setText(m_inputLineEdit->text().replace("/", ""));
  m_response->nativeWidget()->setText(thinkOf(parser->getResponse(m_inputLineEdit->text())));

  QFile f(QDir::homePath() + "/.openbrain/send_to_author");
  QTextStream out(&f);
  if (f.open(QIODevice::Append))
    out << m_inputLineEdit->text() << "\n";
    out << m_response->text() << "\n\n";

  f.close();

  m_inputLineEdit->setText("");
  m_inputLineEdit->setFocus();
}

QString OpenBrain::thinkOf(QString s)
{
  return s;
}

void OpenBrain::createConfigurationInterface(KConfigDialog *parent)
{
    m_settings = new OpenBrainSettings;
    parent->addPage(m_settings, i18n("OpenBrain Settings"), KStandardDirs::locate("icon", "oxygen/scalable/apps/accessories-text-editor.svgz"));
    
    readConfig();
//    m_settings->comboAimlSet-> // TODO: GIVE AIML SETS TO CONFIG UI FROM ~/.openbrain/aiml

    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

void OpenBrain::configAccepted()
{
  KConfigGroup cg = config();

  emit configNeedsSaving();
}

void OpenBrain::readConfig()
{
  KConfigGroup cg = config();

//  m_aimlset = cg.readEntry("aimlset", "default");
  m_font = cg.readEntry("fontstyle", QFont());
  m_color = cg.readEntry("fontcolor", QColor());
}

void OpenBrain::writeConfig()
{
//  m_aimlset = m_settings->comboAimlSet->text();
//  m_font = m_settings->fontStyle->font();
//  m_color = m_settings->fontColor->color();

  KConfigGroup cg = config();

  cg.writeEntry("aimlset", m_aimlset);
  cg.writeEntry("fontstyle", m_font);
  cg.writeEntry("fontcolor",m_color.name());
}

void OpenBrain::loadBrain()
{
  parser = new AIMLParser();

  connect(parser, SIGNAL(aimlLoad_doc(QString)), this, SLOT(updateLoadingPage(QString)));
  connect(parser, SIGNAL(aimlLoad_percent(int)), this, SLOT(updateLoadingPercentage(int)));
  connect(parser, SIGNAL(finished()), this, SLOT(doneLoading()));

  QDir dir(QDir::homePath() + "/.openbrain/aiml/default");
  QStringList files = dir.entryList(QStringList("*.aiml"));
  parser->start();
}

void OpenBrain::displayHome()
{
  m_inputLineEdit->setText("openbrain home display");
  emit returnPressed();
}

void OpenBrain::updateLoadingPage(const QString &str)
{
  m_response->nativeWidget()->setText(str);
}

void OpenBrain::doneLoading()
{
  m_inputLineEdit->setText("openbrain home display");
  emit returnPressed();
  m_inputLineEdit->setFocus();
}

OpenBrain::~OpenBrain()
{
  KConfigGroup cg = config();
  saveState(cg);
}

#include "openbrain.moc"
